import { Controller, Inject, Post, Patch, Body, Res } from '@nestjs/common'
import type { Response, CookieOptions } from 'express'
import { UsersService } from './users.service'
import { ApiTags, ApiOperation } from '@nestjs/swagger'
import { CreateUserDto } from './dto/create-user.dto'
import { LoginInputDto } from './dto/login-input.dto'
import { ResetPassword } from './dto/reset-password.dto'
import { securityConfig, SecurityConfig } from 'src/common/configs'

@Controller('api/users')
@ApiTags('Users')
export class UsersController {
  private refreshTokenCookieConfig: CookieOptions = {
    path: '/api/users/me/token',
    secure: true,
    maxAge: this.secureConfig.refreshIn / 1000,
    httpOnly: true,
    sameSite: 'lax',
  }

  constructor(
    private readonly usersService: UsersService,
    @Inject(securityConfig.KEY) private secureConfig: SecurityConfig,
  ) {}

  @Post()
  @ApiOperation({ summary: '邮箱注册' })
  async registerByEmail(
    @Body() userData: CreateUserDto,
    @Res({ passthrough: true }) res: Response,
  ) {
    const { refreshToken, accessToken } =
      await this.usersService.registerByEmail(userData)
    res.cookie('refreshToken', refreshToken, this.refreshTokenCookieConfig)
    return accessToken
  }

  // TODO: 限制调用频率，避免暴力破解
  @Post('token')
  @ApiOperation({ summary: '邮箱登录' })
  async loginByEmail(
    @Body() user: LoginInputDto,
    @Res({ passthrough: true }) res: Response,
  ) {
    const { refreshToken, accessToken } = await this.usersService.loginByEmail(
      user.email,
      user.password,
    )
    res.cookie('refreshToken', refreshToken, this.refreshTokenCookieConfig)
    return accessToken
  }

  @Patch('password')
  @ApiOperation({ summary: '找回密码' })
  async forgetPassword(
    @Body() payload: ResetPassword,
    @Res({ passthrough: true }) res: Response,
  ) {
    const { refreshToken, accessToken } =
      await this.usersService.resetPasswordByEmail(payload)
    res.cookie('refreshToken', refreshToken, this.refreshTokenCookieConfig)
    return accessToken
  }
}
